import logging

import RhcsQeSdk.core.cli.fabfile as fabfile
from RhcsQeSdk.core.utilities import core_utils

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class User:
    """
    This module provides CLI interface for user management of the Ceph Object Storage service.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " user"

    def create(self, **kw):
        """Create a new user.
        Args:
                kw(dict): Key/value pairs that needs to be provided to the installer
                Example::
                        Supported keys:
                                display-name(str): takes name of the user
                                uid(str): takes the user-id
                                email(str): takes the email address of the user(optional)
                                key-type(str): type of key(optional)
                                access-key(str): takes the access key(optional)
                                secret: takes the secret key(optional)
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """

        cmd = self.base_cmd + " create" + core_utils.build_cmd_args(kw=kw)
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def modify(self, **kw):
        """Modify a user. Typical modifications are to keys and secrets, email addresses,
                display names and access levels.
        Args:
                kw(dict): Key/value pairs that needs to be provided to the installer
                Example::
                Supported keys:
                        uid(str): takes the user-id
                        display-name(str): takes name of the user
                        email(str): takes the email address of the user(optional)
                        key-type(str): type of key(optional)
                        access-key(str): takes the access key(optional)
                        secret|secret-key: takes the secret key(optional)
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " modify" + core_utils.build_cmd_args(kw=kw)
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def info(self, **kw):
        """Display information of a user, and any potentially available subusers and keys.
        Args:
                kw(dict): Key/value pairs that needs to be provided to the installer
        Example::
                Supported keys::
                        uid(str):  takes the user-id
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        uid = kw.get("uid")
        cmd = self.base_cmd + f" info --uid={uid}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rename(self, **kw):
        """Renames a user.
        Args:
                kw(dict): Key/value pairs that needs to be provided to the installer
        Example::
                Supported keys::
                        uid(str):  takes the user-id
                        new-uid(str): takes ID of the new user.
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        uid = kw.get("uid")
        new_uid = kw.get("new-uid")
        cmd = self.base_cmd + f" rename --uid={uid} --new-uid={new_uid}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rm(self, **kw):
        """Remove a user.
        Args:
                kw(dict): Key/value pairs that needs to be provided to the installer
        Example::
                Supported keys::
                        uid(str):  takes the user-id
                        purge-data(bool): This option purges all data associated to the UID(optional)
                        purge-keys(bool): This option purges all keys associated to the UID(optional)
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " rm" + core_utils.build_cmd_args(kw=kw)
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def suspend(self, **kw):
        """Suspend a user.
        Args:
                kw(dict): Key/value pairs that needs to be provided to the installer
        Example::
                Supported keys::
                        uid(str):  takes the user-id
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        uid = kw.get("uid")
        cmd = self.base_cmd + f" suspend --uid={uid}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def enable(self, **kw):
        """Re-enable user after suspension.
        Args:
                kw(dict): Key/value pairs that needs to be provided to the installer
        Example::
                Supported keys::
                        uid(str):  takes the user-id
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        uid = kw.get("uid")
        cmd = self.base_cmd + f" enable --uid={uid}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def check(self, **kw):
        """Check user info.
        Args:
                None
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " check"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def stats(self, **kw):
        """Show user stats as accounted by quota subsystem.
        Args:
                kw(dict): Key/value pairs that needs to be provided to the installer
        Example::
                Supported keys::
                        uid(str):  takes the user-id
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        uid = kw.get("uid")
        cmd = self.base_cmd + f" stats --uid={uid} --sync-stats"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def list_(self, **kw):
        """List all users.
        Args:
                None
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " list"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging

import RhcsQeSdk.core.cli.fabfile as fabfile
from RhcsQeSdk.core.cli.ceph.application import Application

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Pool:
    """
    This module provides CLI interface to manage pool operations of pool within ceph cluster via ceph osd pool.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " pool"
        self.application = Application(self.base_cmd)

    def create(self, **kw):
        """
        The create command will create a pool with provided specifications.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
                pool_name(str): name of the pool.
                pg_num(int): total number of placement groups.
                pgp_num(int): total number of placement groups for placement purpose.
                pool_type(str): pool type can be erasure or replicated.
                crush_rule_name(str): The name of the crush rule for the pool. The rule MUST exist.
                expected_num_objects(int): The expected number of objects for the pool.
                erasure_code_profile(str): A profile when creating an erasure-coded pool and the associated CRUSH rule.

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        pool_name = kw.get("pool_name", "ceph")
        pg_number = kw.get("pg_num", "")
        pgp_number = kw.get("pgp_num", "")
        pool_type = kw.get("pool_type", "")
        crush_rule_name = kw.get("crush_rule_name", "")
        expected_num_objects = kw.get("expected_num_objects", "")
        cmd = f"{self.base_cmd} create {pool_name} {pg_number} {pgp_number} "

        if pool_type == "erasure":
            erasure_code_profile = kw.get("erasure_code_profile", "")
            cmd = (
                cmd
                + f"{pool_type} {erasure_code_profile} {crush_rule_name} {expected_num_objects}"
            )

        else:
            cmd = cmd + f"{pool_type} {crush_rule_name} {expected_num_objects}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def delete(self, **kw):
        """
        The delete command will delete a pool.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
                pool_name(str): name of the pool to be deleted.
                yes-i-really-mean-it(bool): whether to delete or not(Optional)

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        pool_name = kw.get("pool_name", "ceph")
        ending = kw.get("yes-i-really-really-mean-it", "")
        cmd = f"{self.base_cmd} delete {pool_name} {pool_name} --{ending}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rename(self, **kw):
        """
        The rename command will rename a pool.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
                pool_name(str): current name of a pool.
                new_pool_name(str): new name to be given to the pool.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return vlaue for that host's execution run.
        """
        kw = kw.get("kw")
        current_pool_name = kw.get("pool_name", "ceph")
        new_pool_name = kw.get("new_pool_name", "")
        cmd = f"{self.base_cmd} rename {current_pool_name} {new_pool_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def set_quota(self, **kw):
        """
        The command set-quota can be used to set the values of max_objects and max_bytes of a pool.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example:
            Supported Keys:
                pool_name(str): name of the pool.
                max_objects(str): to check whether object count has to be set.
                max_bytes(str): to check whether byte count has to be set.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        pool_name = kw.get("pool_name", "ceph")
        max_objects = kw.get("max_objects", None)
        max_bytes = kw.get("max_bytes", None)
        cmd = f"{self.base_cmd} set-quota {pool_name} "

        if max_bytes and max_objects:
            cmd = cmd + f" max_objects max_objects {max_objects} max_bytes {max_bytes}"
        elif max_objects:
            cmd = cmd + f" max_objects {max_objects}"
        else:
            cmd = cmd + f" max_bytes {max_bytes}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def get_quota(self, **kw):
        """
        The command get-quota can be used to retrieve the values of max_objects and max_bytes of a pool.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
                pool_name(str): name of the pool.
                max_objects(str): to check whether object count has to be retrieved.
                max_bytes(str): to check whether byte count has to be retrieved.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        pool_name = kw.get("pool_name", "ceph")
        max_objects = kw.get("max_objects", None)
        max_bytes = kw.get("max_bytes", None)
        cmd = f"{self.base_cmd} get-quota {pool_name} "

        if max_bytes and max_objects:
            cmd = cmd + f" max_objects max_objects {max_objects} max_bytes {max_bytes}"
        elif max_objects:
            cmd = cmd + f" max_objects {max_objects}"
        else:
            cmd = cmd + f" max_bytes {max_bytes}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def mksnap(self, **kw):
        """
        The command mksnap will make a snapshot in the pool specified by the user.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:

            pool_name(str): name of the pool.
            snap_name(str): name of the snapshot.

        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        pool_name = kw.get("pool_name", "ceph")
        snap_name = kw.get("snap_name", "")
        cmd = f"{self.base_cmd} mksnap {pool_name} {snap_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rmsnap(self, **kw):
        """
        The command rmsnap will remove a snap shot from the pool specified by the user.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
                pool_name(str): name of the pool.
                snap_name(str): name of the snapshot to be removed.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        pool_name = kw.get("pool_name", "ceph")
        snap_name = kw.get("snap_name", "")
        cmd = f"{self.base_cmd} rmsnap {pool_name} {snap_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def get_(self, **kw):
        """
        The command get will retrun value of a key as specified by the user.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
                pool_name(str): name of the pool
                key(Union[str,int]): name of the property of an osd pool.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        pool_name = kw.get("pool_name", "ceph")
        key = kw.get("key", None)
        cmd = f"{self.base_cmd} get {pool_name} {key}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def set_(self, **kw):
        """
        The command set will set a value to a particular key in pool.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
                pool_name(str): name of the pool.
                key(Union[str,int]): name of the property of an osd pool
                value(Union[str,int]): vlaue to be set against the key.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        pool_name = kw.get("pool_name", "ceph")
        key = kw.get("key", None)
        value = kw.get("value", None)
        cmd = f"{self.base_cmd} set {pool_name} {key} {value}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging
from copy import deepcopy

import RhcsQeSdk.core.cli.fabfile as fabfile
from RhcsQeSdk.core.utilities import core_utils

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Namespace:

    """
    This Class provides wrappers for rbd namespace commands.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " namespace"

    def create(self, **kw):
        """Wrapper for rbd namespace create.

        Create an rbd image namespace inside specified pool.
        Args:
        kw: Key value pair of method arguments
            Example::
            Supported keys:
                pool_spec(str): pool specification <pool-name>[/<namespace>]
                namespace(str): name of the namespace
                pool(str): pool name
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")

        kw_copy = deepcopy(kw)
        pool_spec = kw_copy.pop("pool_spec", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)

        cmd = f"{self.base_cmd} create {pool_spec}{cmd_args}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def list_(self, **kw):
        """Wrapper for rbd namespace list.

        List RBD image namespaces.
        Args:
        kw: Key value pair of method arguments
            Example::
            Supported keys:
                pool_spec(str): pool specification.
                pool(str): pool name
                format(str): output format (plain, json, or xml) [default: plain]
                pretty-format(bool): True
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")

        kw_copy = deepcopy(kw)
        pool_spec = kw_copy.pop("pool_spec", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)

        cmd = f"{self.base_cmd} list {pool_spec}{cmd_args}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def remove_(self, **kw):
        """Wrapper for rbd namespace remove.

        Remove an RBD image namespace.
        Args:
        kw: Key value pair of method arguments
            Example::
            Supported keys:
                pool_spec(str): pool specification <pool-name>[/<namespace>]
                namespace(str): name of the namespace
                pool(str): pool name
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")

        kw_copy = deepcopy(kw)
        pool_spec = kw_copy.pop("pool_spec", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)

        cmd = f"{self.base_cmd} remove {pool_spec}{cmd_args}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



from RhcsQeSdk.core.cli.ceph.ceph import Ceph
from RhcsQeSdk.core.cli.radosgw_admin.radosgw_admin import Radosgw_admin

kw = {
    "rgw-realm": "realm1",
    "rgw-zonegroup": "zonegroup1",
    "rgw-zone": "zone1",
    "access-key": None,
    "secret-key": None,
    "endpoints": None,
}

kwargs = {
    "service_type": "rgw",
    "service_name": "test",
    "realm": "realm1",
    "zone": "zone1",
    "placement": "'1 admin'",
}

rgw = Radosgw_admin()
ceph = Ceph()
rgw.realm.create(**kw)
rgw.zonegroup.create({"rgw-zonegroup": "zonegroup1", "default": True, "master": True})
rgw.zone.create(**kw)
rgw.zone.create({"rgw-zonegroup": "zonegroup1", "rgw-zone": "zone2"})
ceph.orch.apply(kwargs)
rgw.period.commit(**kw)
rgw.zonegroup.zonegroup_remove({"rgw-zonegroup": "zonegroup1", "rgw-zone": "zone2"})
rgw.period.commit(**kw)
rgw.zone.delete({"rgw-zone": "zone2"})
rgw.period.commit(**kw)
rgw.zonegroup.zonegroup_remove(**kw)
rgw.period.commit(**kw)
rgw.zonegroup.delete(**kw)
rgw.realm.delete(**kw)



class Metadata:
    _instance = None
    ceph_cluster_dict = None
    cluster_name = None
    ceph = None
    method = None
    roles = None
    must_present_key = None
    must_method = None
    kwargs = None
    env_config = {
        "hosts": None,
        "username": None,
        "password": None,
        "parallel": True,
        "use_ssh_config": True,
        "use_shell": False,
        "timeout": 300,
        "sudo": False,
    }
    suite_output = {}

    def __new__(
        cls,
        ceph_cluster_dict=None,
        cluster_name=None,
        ceph=None,
        method=None,
        roles=None,
        must_present_key=None,
        kwargs=None,
        parallel=None,
        use_ssh_config=None,
        use_shell=None,
        timeout=None,
        sudo=None,
        hosts=None,
        must_method=None,
        suite_output=None,
    ):
        if not Metadata._instance:
            Metadata._instance = cls
        if ceph_cluster_dict:
            Metadata._instance.ceph_cluster_dict = ceph_cluster_dict
        if cluster_name:
            Metadata._instance.cluster_name = cluster_name
        if ceph:
            Metadata._instance.ceph = ceph
        if method:
            Metadata._instance.method = method
        if roles:
            Metadata._instance.roles = roles
        if kwargs:
            Metadata._instance.kwargs = kwargs
        if must_present_key:
            Metadata._instance.must_present_key = must_present_key
        if parallel:
            Metadata._instance.env_config["parallel"] = parallel
        if use_ssh_config:
            Metadata._instance.env_config["use_ssh_config"] = use_ssh_config
        if use_shell:
            Metadata._instance.env_config["use_shell"] = use_shell
        if timeout:
            Metadata._instance.env_config["timeout"] = timeout
        if sudo:
            Metadata._instance.env_config["sudo"] = sudo
        if hosts:
            Metadata._instance.env_config["hosts"] = hosts
        if must_method:
            Metadata._instance.must_method = must_method
        if suite_output:
            Metadata._instance.suite_output = suite_output
        return Metadata._instance



import logging

import RhcsQeSdk.core.cli.fabfile as fabfile

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Subscription:
    """
    This module provides CLI interface to manage pubsub subscription.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " subscription"

    def get_(self, **kw):
        """
        This method is used to get a pubsub subscription definition.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              topic-name(str): The pubsub subscription name.
              tenant(str): The bucket notifications/pubsub topic name.

        Returns:
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        topic_name = kw.get("topic-name")
        tenant = kw.get("tenant")
        cmd = self.base_cmd + f" get --subscription={topic_name} --tenant={tenant}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rm(self, **kw):
        """
        This method is used to remove a pubsub subscription.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              topic-name(str): The pubsub subscription name.
              tenant(str): The bucket notifications/pubsub topic name.

        Returns:
          Dict(str): A mapping of host strings to the given task’s return value for that host’s execution run.
        """
        kw = kw.get("kw")
        topic_name = kw.get("topic-name")
        tenant = kw.get("tenant")
        cmd = self.base_cmd + f" rm --subscription={topic_name} --tenant={tenant}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def pull(self, **kw):
        """
        This method is used to show events in a pubsub subscription.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              topic-name(str): The pubsub subscription name.
              tenant(str): The bucket notifications/pubsub topic name.
              last-marker(str): To resume if command gets interrupted.

        Returns:
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        topic_name = kw.get("topic-name")
        last_marker = kw.get("last-marker")
        tenant = kw.get("tenant")
        cmd = (
            self.base_cmd
            + f" pull --subscription={topic_name} --marker={last_marker} --tenant={tenant}"
        )
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def ack(self, **kw):
        """
        This method is used to ack (remove) an event in a pubsub subscription.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              topic-name(str): The pubsub subscription name.
              tenant(str): The bucket notifications/pubsub topic name.
              event-id(str): The event id in a pubsub subscription.

        Returns:
          Dict(str): A mapping of host strings to the given task’s return value for that host’s execution run.
        """
        kw = kw.get("kw")
        topic_name = kw.get("topic-name")
        event_id = kw.get("event-id")
        tenant = kw.get("tenant")
        cmd = (
            self.base_cmd
            + f" ack --subscription={topic_name} --event-id={event_id} --tenant={tenant}"
        )
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging

import RhcsQeSdk.core.cli.fabfile as fabfile
from RhcsQeSdk.core.cli.ceph.module import Module

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Mgr:
    """
    This module provides CLI interface to manage the MGR service.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " mgr"
        self.module = Module(self.base_cmd)

    def services(self, **kw):
        """
        This method is used to see the addresses of modules with IP and Port number.
        Args:
          None

         Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " services"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging

import RhcsQeSdk.core.cli.fabfile as fabfile

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class ImageMeta:
    """
    This module provides CLI interface to manage block device image metadata.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " image-meta"

    def set_(self, **kw):
        """
        This method is used to set a new metadata key-value pair.
        Args:
          kw(dict): Key/value pairs that needs to be provided to the installer
          Example:
            Supported keys:
              pool_name(str): name of the pool.
              image_name(str): name of the image.
              key(str): name of the metadata key.
              value(str): value corresponding to the metadata key.

        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        image_name = kw.get("image_name")
        pool_name = kw.get("pool_name")
        key = kw.get("key")
        value = kw.get("value")
        cmd = self.base_cmd + " set" + f" {pool_name}/{image_name} {key} {value}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def remove_(self, **kw):
        """
        This method is used to remove a metadata key-value pair.
        Args:
          kw(dict): Key/value pairs that needs to be provided to the installer
          Example:
            Supported keys:
              pool_name(str): name of the pool.
              image_name(str): name of the image.
              key(str): name of the metadata key.

        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        image_name = kw.get("image_name")
        pool_name = kw.get("pool_name")
        key = kw.get("key")
        cmd = self.base_cmd + " remove" + f" {pool_name}/{image_name} {key}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def get_(self, **kw):
        """
        This method is used to view the value of a key.
        Args:
          kw(dict): Key/value pairs that needs to be provided to the installer
          Example:
            Supported keys:
              pool_name(str): name of the pool.
              image_name(str): name of the image.
              key(str): name of the metadata key.

        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        image_name = kw.get("image_name")
        pool_name = kw.get("pool_name")
        key = kw.get("key")
        cmd = self.base_cmd + " get" + f" {pool_name}/{image_name} {key}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def list_(self, **kw):
        """
        This method is used to show all metadata on an image.
        Args:
          kw(dict): Key/value pairs that needs to be provided to the installer
          Example:
            Supported keys:
              pool_name(str): name of the pool.
              image_name(str): name of the image.

        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        image_name = kw.get("image_name")
        pool_name = kw.get("pool_name")
        cmd = self.base_cmd + " list" + f" {pool_name}/{image_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging
from copy import deepcopy

import RhcsQeSdk.core.cli.fabfile as fabfile
from RhcsQeSdk.core.utilities import core_utils

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Schedule:
    """
    This module provides CLI interface to manage the mirror snapshot scheduling.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " schedule"

    def add_(self, **kw):
        """
        This method is used to create a mirror-snapshot schedule.
        Args:
          kw(dict): Key/value pairs that needs to be provided to the installer
          Example:
            Supported keys:
              pool_name(str): name of the pool.
              image_name(str): name of the image.
              cluster(str): name of the cluster(Optional).
              start_time(str): can be specified using the ISO 8601 time format(Optional).

        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        kw_copy = deepcopy(kw)
        image_name = kw_copy.pop("image_name")
        pool_name = kw_copy.pop("pool_name")
        interval = kw_copy.pop("interval")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)
        cmd = (
            self.base_cmd
            + " add"
            + f" --pool {pool_name} --image {image_name} {interval}"
            + cmd_args
        )
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def ls(self, **kw):
        """
        This method is used to list all snapshot schedules for a specific level (global, pool, or image).
        Args:
          kw(dict): Key/value pairs that needs to be provided to the installer
          Example:
            Supported keys:
              pool_name(str): name of the pool(Optional).
              image_name(str): name of the image(Optional).
              cluster(str): name of the cluster(Optional).
              recursive(bool): set this to true to list all schedules at the specified level and below.

        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        cmd_args = core_utils.build_cmd_args(kw=kw)
        cmd = self.base_cmd + " ls" + cmd_args
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def status(self, **kw):
        """
        This method is used to view the status for when the next snapshots will be created for snapshot based mirroring.
        Args:
          kw(dict): Key/value pairs that needs to be provided to the installer
          Example:
            Supported keys:
              pool_name(str): name of the pool(Optional).
              image_name(str): name of the image(Optional).

        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        cmd_args = core_utils.build_cmd_args(kw=kw)
        cmd = self.base_cmd + " status" + cmd_args
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def remove_(self, **kw):
        """
        This method is used to remove a mirror-snapshot schedule.
        Args:
          kw(dict): Key/value pairs that needs to be provided to the installer
          Example:
            Supported keys:
              pool_name(str): name of the pool.
              image_name(str): name of the image.
              cluster(str): name of the cluster(Optional).
              start_time(str): can be specified using the ISO 8601 time format(Optional).

        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        kw_copy = deepcopy(kw)
        image_name = kw_copy.pop("image_name")
        pool_name = kw_copy.pop("pool_name")
        interval = kw_copy.pop("interval")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)
        cmd = (
            self.base_cmd
            + " remove"
            + f" --pool {pool_name} --image {image_name} {interval}"
            + cmd_args
        )
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging
from copy import deepcopy

import RhcsQeSdk.core.cli.fabfile as fabfile
from RhcsQeSdk.core.utilities import core_utils

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Topic:
    """
    This module provides CLI interface for topic management of object gateway service.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " topic"

    def list_(self, **kw):
        """
        This method is used to list bucket notifications/pubsub topics.
        Args:
        kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              tenant(str): The bucket notifications/pubsub topic name.

        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " list" + core_utils.build_cmd_args(kw=kw)
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def get_(self, **kw):
        """
        This method is used to get a bucket notifications/pubsub topic.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              topic-name(str): The pubsub subscription name.
              tenant(str): The bucket notifications/pubsub topic name.

        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        kw_copy = deepcopy(kw)
        topic_name = kw_copy.pop("topic-name", "")
        cmd = (
            self.base_cmd
            + f" get --topic={topic_name}"
            + core_utils.build_cmd_args(kw=kw_copy)
        )
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rm(self, **kw):
        """
        This method is used to remove a bucket notifications/pubsub topic.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              topic-name(str): The pubsub subscription name.
              tenant(str): The bucket notifications/pubsub topic name.

        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        kw_copy = deepcopy(kw)
        topic_name = kw_copy.pop("topic-name", "")
        cmd = (
            self.base_cmd
            + f" rm --topic={topic_name}"
            + core_utils.build_cmd_args(kw=kw_copy)
        )
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



"""Module for fabric operations."""
import logging

from fabric.api import env, execute, put, run, settings, sudo

from RhcsQeSdk.core.utilities.metadata import Metadata

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)

file_handler = logging.FileHandler("startup.log", mode="a")
file_handler.setLevel(logging.ERROR)

file_handler.setFormatter(formatter)
logger.addHandler(file_handler)


class CommandException(Exception):  # Should be your Exception
    pass


class FabricException(Exception):
    pass


def run_command(cmd, config=None):
    """
    Execute a command on given list of remote machines using Fabric.

    Attributes:
        cmd(str): Command to execute in remote node
        config(Dict): dictionary of environmental attributes to run command in fabfile env.

    Returns:
        list(str)
        out: output after execution of command
    """
    # If you don't want to abort for specific exception and handle it with an implementation by keeping warn_only=False
    env.abort_exception = FabricException

    config = config or Metadata().env_config
    env.hosts = config.get("hosts")
    env.user = config.get("username", "")
    env.password = config.get("password", "")
    env.timeout = config.get("timeout", 10)  # default is 10s
    env.key_filename = ""
    env.connection_attempts = config.get("connection_attempts", 1)  # default is 1
    env.keepalive = config.get("keepalive", 0)  # for long_running and default is 0
    env.parallel = config.get("parallel", False)
    env.use_shell = config.get("use_shell", False)
    env.host_string = config.get("hosts")
    env.shell = config.get("shell", False)
    with settings(warn_only=True, combine_stderr=False):
        if env.parallel:
            out = execute(execute_run_cmd, cmd, config)
            for ip, result in out.items():
                parse_output(result, cmd)
            return out
        else:
            output_dict = dict()
            logger.info(f"Running {cmd}")
            if config.get("sudo"):
                output_dict["ip"] = parse_output(sudo(cmd))
            else:
                output_dict["ip"] = parse_output(run(cmd))
            return output_dict


def execute_run_cmd(cmd, config=None):
    """
    Execute a command as a metatask on given list of remote machines using Fabric
    in parallel.

    Attributes:
        cmd(str): Command to execute in remote node
        config(Dict): dictionary of environmental attributes to run command in fabfile env.

    Returns:
        list(str)
        out: output after execution of command
    """
    logger.info(f"Running {cmd}")
    if config.get("sudo"):
        with settings(
            warn_only=True, combine_stderr=False, sudo_user="root", sudo_prefix="sudo"
        ):
            return sudo(cmd)
    else:
        with settings(warn_only=True, combine_stderr=False):
            return run(cmd)


def execute_put_cmd(source_path, destination_path, config=None):
    """
    Downloads a file as a metatask on given list of remote machines using Fabric
    in parallel.

    Attributes:
        source_path(str): source path of content to be transferred.
        destination_path(str): destination path of content into which should be transferred.
        config(Dict): dictionary of environmental attributes to run command in fabfile env.

    Returns:
        list(str)
        out: output after execution of command
    """
    if config.get("sudo"):
        with settings(
            warn_only=True, combine_stderr=False, sudo_user="root", sudo_prefix="sudo"
        ):
            return put(source_path, destination_path, use_sudo=True)
    else:
        with settings(warn_only=True, combine_stderr=False):
            out = put(source_path, destination_path)
            parse_output(out)
            return out


def put_command(source_path, destination_path, config):
    """
    To overwrite remote files on given list of remote machines using Fabric in serial.

    Attributes:
        cmd(str): Command to execute in remote node

    Returns:
        list(str)
        out: output after execution of command
    """
    # If you don't want to abort for specific exception and handle it with an implementation by keeping warn_only=False
    env.abort_exception = FabricException

    env.hosts = config.get("hosts")
    env.user = config.get("username", "cephuser")
    env.password = config.get("password", "cephuser")
    env.timeout = config.get("timeout", 10)  # default is 10s
    env.key_filename = ""
    env.connection_attempts = config.get("connection_attempts", 1)  # default is 1
    env.keepalive = config.get("keepalive", 0)  # for long_running and default is 0
    env.parallel = config.get("parallel", False)
    env.host_string = config.get("hosts")
    with settings(warn_only=True, combine_stderr=False):
        if env.parallel:
            return execute(execute_put_cmd, source_path, destination_path, config)
        else:
            output_dict = dict()
            if config.get("sudo"):
                output_dict["ip"] = parse_output(
                    put(source_path, destination_path, use_sudo=True)
                )
            else:
                output_dict["ip"] = parse_output(put(source_path, destination_path))
            return output_dict


def parse_output(result, cmd=None):
    """
    Parses output and error from result of command.

    Attributes:
        cmd(str): Command to execute in remote node
        result(str): Result of command execution.

    Returns:
        list(str)
        result: output after execution of command

    Raises:
        Exception
    """
    return result



import logging

from RhcsQeSdk.core.cli.rbd.mirror.image import Image
from RhcsQeSdk.core.cli.rbd.mirror.pool import Pool
from RhcsQeSdk.core.cli.rbd.mirror.snapshot import Snapshot

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Mirror:
    """Base class for all rbd mirror commands.

    This module provides CLI interface to manage rbd mirror and
    objects with wrapper for sub-commands.

    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " mirror"
        self.image = Image(self.base_cmd)
        self.pool = Pool(self.base_cmd)
        self.snapshot = Snapshot(self.base_cmd)



import logging

import RhcsQeSdk.core.cli.fabfile as fabfile

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Balancer:
    """
    This module provides CLI interface to manage ceph balancer operations.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " balancer"

    def status(self, **kw):
        """Currently recorded mode, any plans, whether it's enabled are shown as part of the status command.
        Args:
                None
        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " status"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def mode(self, **kw):
        """Adjusts the mode.
        Args:
                kw(dict): Key/value pairs that needs to be provided to the installer
        Example::
        Supported keys:
                mode(str): takes the optimation method to use <upmap,crush-compat>
        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        mode = kw.get("mode")
        cmd = self.base_cmd + f" mode {mode}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def on(self, **kw):
        """Turns the balancer on.
        Args:
                None
        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " on"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def off(self, **kw):
        """Turns the balancer off.
        Args:
                None
        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " off"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def eval_balancer(self, **kw):
        """Evaluats the quality of the data distribution, either for the current
           PG distribution, or the PG distribution that would result after executing a plan.
        Args:
                kw(dict): Key/value pairs that needs to be provided to the installer
        Example::
        Supported keys:
                pool-name(str): takes the name of the pool (optional)
                verbose(bool): to get the greater detail for the evaluation (optional)
                plan-name(str): takes the name of the plan (optional)
        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " eval"
        pool_name = kw.get("pool-name")
        verbose = kw.get("verbose")
        plan_name = kw.get("plan-name")
        if pool_name:
            cmd = cmd + f" {pool_name}"
        elif plan_name:
            cmd = cmd + f" {plan_name}"
        elif verbose:
            cmd = cmd + "-verbose ..."
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def optimize(self, **kw):
        """This is used to optimize named <plan> based on the current mode.
        Args:
                kw(dict): Key/value pairs that needs to be provided to the installer
        Example::
        Supported keys:
                plan-name(str): takes the name of the plan
        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        plan_name = kw.get("plan-name")
        cmd = self.base_cmd + f" optimize {plan_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def show(self, **kw):
        """shows what the plan would do (basically a dump of cli commands to
               adjust weights etc).
        Args:
                kw(dict): Key/value pairs that needs to be provided to the installer
        Example::
        Supported keys:
                plan-name(str): takes the name of the plan
        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        plan_name = kw.get("plan-name")
        cmd = self.base_cmd + f" show {plan_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def ls(self, **kw):
        """Display all the plans.
        Args:
                None
        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " ls"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rm(self, **kw):
        """Old plans can be discarded with this command.
        Args:
                kw(dict): Key/value pairs that needs to be provided to the installer
        Example::
        Supported keys:
                plan-name(str): takes the name of the plan
        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        plan_name = kw.get("plan-name")
        cmd = self.base_cmd + f" rm {plan_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def execute_balancer(self, **kw):
        """execute plan (and then discard it).
        Args:
                kw(dict): Key/value pairs that needs to be provided to the installer
        Example::
        Supported keys:
                plan-name(str): takes the name of the plan
        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        plan_name = kw.get("plan-name")
        cmd = self.base_cmd + f" execute {plan_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



from RhcsQeSdk.core.utilities.test_config_run import ConfigRun
from RhcsQeSdk.core.utilities.utils import (
    create_file,
    read_from_file,
    sleep,
    write_to_file,
)


def run(kw):
    test_data = kw.get("test_data")
    cluster_name = kw.get("cluster_name")
    output = {"cluster_names": [], "rc": 1}
    if cluster_name:
        output["cluster_names"].append(cluster_name)
    for step in test_data.get("configs"):
        for role in step.get("role"):
            if len(role.split(":")) == 2:
                output["cluster_names"].append(role.split(":")[0])
    ceph_cluster_dict = kw.get("ceph_cluster_dict")
    steps = test_data.get("configs")
    step0_cluster_name = steps[0].get("role")[0].split(":")[0]
    output["cluster_names"].append(step0_cluster_name)
    ips = []
    nodes = ceph_cluster_dict[step0_cluster_name].get_nodes("client")
    for node in nodes:
        ips.append(node.ip_address)
    env_config = {
        "hosts": ips,
        "username": "cephuser",
        "password": "cephuser",
        "parallel": True,
    }
    kw = {
        "file_name": "bootstrap_token_master",
        "path": "/home/cephuser",
        "env_config": env_config,
    }
    create_file(kw=kw)
    if not ConfigRun(step0_cluster_name, ceph_cluster_dict).run_step(steps[0]):
        return output
    kw = {
        "sleep_time": 30,
    }
    sleep(kw=kw)

    kw = {
        "file_name": "bootstrap_token_master",
        "path": "/home/cephuser",
        "env_config": env_config,
    }
    token = list(read_from_file(kw=kw).values())[0]
    token_path = steps[1].get("args").get("token_path")
    step1_cluster_name = steps[1].get("role")[0].split(":")[0]
    role = steps[1].get("role")[0].split(":")[1]
    ips = []
    nodes = ceph_cluster_dict[step1_cluster_name].get_nodes(role)
    for node in nodes:
        ips.append(node.ip_address)
    env_config = {
        "hosts": ips,
        "username": "cephuser",
        "password": "cephuser",
        "parallel": True,
    }
    kw = {
        "file_name": "bootstrap_token_master",
        "path": "/home/cephuser",
        "env_config": env_config,
    }
    create_file(kw=kw)
    kw = {
        "content": token,
        "path": token_path,
        "env_config": env_config,
    }
    write_to_file(kw=kw)
    if not ConfigRun(step1_cluster_name, ceph_cluster_dict).run_step(steps[1]):
        return output

    kw = {
        "sleep_time": 30,
    }
    sleep(kw=kw)
    output["rc"] = 0
    output["cluster_names"].append(step1_cluster_name)
    return output



import logging
from copy import deepcopy

import RhcsQeSdk.core.cli.fabfile as fabfile
from RhcsQeSdk.core.cli.ceph.orch.daemon import Daemon
from RhcsQeSdk.core.cli.ceph.orch.host import Host
from RhcsQeSdk.core.cli.ceph.orch.osd import Osd
from RhcsQeSdk.core.cli.ceph.orch.upgrade import Upgrade
from RhcsQeSdk.core.utilities import core_utils

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Orch:
    """
    This module provides a command line interface (CLI) to ceph orch.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " orch"
        self.host = Host(self.base_cmd)
        self.daemon = Daemon(self.base_cmd)
        self.osd = Osd(self.base_cmd)
        self.upgrade = Upgrade(self.base_cmd)

    def status(self, **kw):
        """
        Displays the status of orchestrator.
        Args:
            None
        Returns:
          Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " status"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def ls(self, **kw):
        """
        Lists the status of one of the services running in the Ceph cluster.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              service_type(str): type of service(mon, osd, mgr, mds, rgw)(Optional).
              service_name(str): name of host(Optional).
              export(bool): Whether to export the service specifications knows to the orchestrator(Optional).
              format(str): the type to be formatted(yaml)(Optional).
              refresh(bool): Whether to refresh or not(Optional).
        Returns:
          Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " ls" + core_utils.build_cmd_args(kw=kw)
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def ps(self, **kw):
        """
        Lists daemons known to orchestrator.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              daemon_type(str): type of daemon(mon, osd, mgr, mds, rgw)(Optional).
              daemon_id(int): id of daemon(mon, osd, mgr, mds, rgw)(Optional).
              hostname(str): name of host(Optional).
              service_name(str): name of service(Optional).
              format(str): the type to be formatted(yaml)(Optional).
              refresh(bool): Whether to refresh or not(Optional).
        Returns:
          Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " ps" + core_utils.build_cmd_args(kw=kw)
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def apply(self, **kw):
        """
        Applies the configuration to hosts.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              service-name(str): name of service.
              hosts(list): name of host(Optional).
              placement(dict): Placement on hosts(Optional).
              count(int): No of hosts to be applied(Optional)
        Returns:
          Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        kw_copy = deepcopy(kw)
        service_name = kw_copy.pop("service_name", "")
        cmd = f"{self.base_cmd} apply {service_name}"
        nodes_str = kw_copy.pop("nodes_str", "")
        label = kw_copy.pop("label", "")
        if label:
            cmd = cmd + f" --placement='label:{label}'"
        if nodes_str:
            cmd = cmd + f"{nodes_str}"
        cmd = cmd + core_utils.build_cmd_args(kw=kw_copy)
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rm(self, **kw):
        """
        Removes a service.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              service_name(str): name of service.
        Returns:
          Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        service_name = kw.get("service_name")
        cmd = self.base_cmd + f" rm {service_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def set_(self, **kw):
        """
        Sets/unsets a module to configure as backend.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              Module(str): Module to configure as backend.
        Returns:
          Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        module = kw.get("module") if kw.get("module") else ""
        cmd = self.base_cmd + f" set backend {module}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def start(self, **kw):
        """
        Starts a service.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              service_name(str): name of service.
        Returns:
          Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        service_name = kw.get("service_name")
        cmd = self.base_cmd + f" start {service_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def stop(self, **kw):
        """
        Stops a service.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              service_name(str): name of service.
        Returns:
          Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        service_name = kw.get("service_name")
        cmd = self.base_cmd + f" stop {service_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def restart(self, **kw):
        """
        Restarts a service.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              service_name(str): name of service.
        Returns:
          Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        service_name = kw.get("service_name")
        cmd = self.base_cmd + f" restart {service_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def reconfig(self, **kw):
        """
        Reconfigs a service.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              service_name(str): name of service.
        Returns:
          Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        service_name = kw.get("service_name")
        cmd = self.base_cmd + f" reconfig {service_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def redeploy(self, **kw):
        """
        Redeploys a service.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              service_name(str): name of service.
        Returns:
          Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        service_name = kw.get("service_name")
        cmd = self.base_cmd + f" redeploy {service_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging
from copy import deepcopy

import RhcsQeSdk.core.cli.fabfile as fabfile
from RhcsQeSdk.core.utilities import core_utils

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Role:
    """
    This module provides CLI interface to manage role operations.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " role"

    def create(self, **kw):
        """Create a new AWS role for use with STS.
        Args:
                kw(dict): Key/value pairs that needs to be provided to the installer
        Example::
        Supported keys:
                role-name(str): takes name of the role
                path(str): path to role. The default value is a slash(/).(optioal)
                assume-role-policy-doc(str): The trust relationship policy document that grants an
                                             entity permission to assume the role.(optional)
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        kw_copy = deepcopy(kw)
        role_name = kw_copy.pop("role-name", "")
        cmd = (
            self.base_cmd
            + f" create --role-name={role_name}"
            + core_utils.build_cmd_args(kw=kw_copy)
        )
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rm(self, **kw):
        """Remove a role.
        Args:
                kw(dict): Key/value pairs that needs to be provided to the installer
        Example::
        Supported keys:
                role-name(str): takes name of the role
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        role_name = kw.get("role-name")
        cmd = self.base_cmd + f" rm --role-name={role_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def get_(self, **kw):
        """Get a role.
        Args:
                kw(dict): Key/value pairs that needs to be provided to the installer
        Example::
        Supported keys:
                role-name(str): takes name of the role
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        role_name = kw.get("role-name")
        cmd = self.base_cmd + f" get --role-name={role_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def list_(self, **kw):
        """List the roles with specified path prefix.
        Args:
          kw(dict): Key/value pairs that needs to be provided to the installer
          Example:
          Supported keys:
                path-prefix(str): Path prefix for filtering roles. If this is not specified,
                                  all roles are listed.(optional)
        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " list" + core_utils.build_cmd_args(kw=kw)
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def modify(self, **kw):
        """Modify the assume role policy of an existing role.
        Args:
                kw(dict): Key/value pairs that needs to be provided to the installer
        Example::
        Supported keys:
                role-name(str): takes name of the role
                assume-role-policy-doc(str): The trust relationship policy document that grants an
                                             entity permission to assume the role.
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        role_name = kw.get("role-name")
        trust_policy_document = kw.get("assume-role-policy-doc")
        cmd = (
            self.base_cmd
            + f" modify --role-name={role_name} --assume-role-policy-doc={trust_policy_document}"
        )
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging
from copy import deepcopy

import RhcsQeSdk.core.cli.fabfile as fabfile
from RhcsQeSdk.core.cli.rbd.mirror.bootstrap import Bootstrap
from RhcsQeSdk.core.utilities import core_utils

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Peer:
    """This module provides CLI interface to manage rbd mirror pool commands."""

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " peer"
        self.bootstrap = Bootstrap(self.base_cmd)

    def add_(self, **kw):
        """Wrapper for rbd mirror pool peer add.

        Command is used for adding a mirror pool peer
        Args:
            kw: Key value pair of method arguments
            Example::
            Supported keys:
                pool_name: Name of the pool.
                pool_name: Name of the pool. (--pool <name>)
                remote_cluster_spec: remote cluster specification - <client-name>@<cluster-name>
                remote_client_name: remote client name
                remote_cluster: remote cluster name
                remote_mon_host: remote mon host(s)
                remote_key_file: remote key file path
                direction: mirroring direction (rx-only, rx-tx)
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")

        kw_copy = deepcopy(kw)
        pool_name = kw_copy.pop("pool_name", "")
        remote_cluster_spec = kw_copy.pop("remote_cluster_spec", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)

        cmd = f"{self.base_cmd} add {pool_name} {remote_cluster_spec}{cmd_args}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def remove_(self, **kw):
        """Wrapper for rbd mirror pool peer remove.

        Command is used to remove a mirroring peer from a pool.

        Args:
            kw: Key value pair of method arguments
            Example::
            Supported keys:
                pool_name: Name of the pool.
                pool_name: Name of the pool. (--pool <name>)
                uuid: peer uuid

        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")

        kw_copy = deepcopy(kw)
        pool_name = kw_copy.pop("pool_name", "")
        uuid = kw_copy.pop("uuid", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)

        cmd = f"{self.base_cmd} remove {pool_name} {uuid}{cmd_args}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def set_(self, **kw):
        """Wrapper for rbd mirror pool peer set.

        This command is used to update mirroring peer settings.

        Args:
            kw: Key value pair of method arguments
            Example::
            Supported keys:
                pool_name: Name of the pool.
                pool_name: Name of the pool. (--pool <name>)
                uuid: peer uuid
                key: peer parameter
                (direction, site-name, client, mon-host, key-file)
                value: new value for specified key

        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")

        kw_copy = deepcopy(kw)
        pool_name = kw_copy.pop("pool_name", "")
        uuid = kw_copy.pop("uuid", "")
        key = kw_copy.pop("key", "")
        value = kw_copy.pop("value", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)

        cmd = f"{self.base_cmd} set {pool_name} {uuid} {key} {value}{cmd_args}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging

import RhcsQeSdk.core.cli.fabfile as fabfile

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Health:
    """This module provides CLI interface to implement ceph health associated commands."""

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " health"

    def detail(self, **kw):
        """
        The command detail will display the clusters health in detail.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:

            None

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " detail"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def unmute(self, **kw):
        """
        The command unmute will unmute a type of health check.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:

            code(str): the health check code.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        code = kw.get("code", "")
        cmd = self.base_cmd + f" unmute {code}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def mute(self, **kw):
        """
        The command mute will mute a type of health check.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:

            code(str): the health check code.
            muting_time(str): time duration the health check code has to be muted.(optional)

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        code = kw.get("code", "")
        muting_time = kw.get("muting_time", "")
        cmd = self.base_cmd + f" mute {code} {muting_time}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging

import RhcsQeSdk.core.cli.fabfile as fabfile
from RhcsQeSdk.core.utilities import core_utils

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Quota:
    """
    This module provides CLI interface for quota management on users and buckets
    owned by users of the object gateway service.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " quota"

    def set_(self, **kw):
        """
        Sets quota parameters, before you enable it.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              quota-scope(str): Scope for quota in either bucket/user.
              uid(int): uid of an user.
              num-objects(int): maximum number of objects(Optional).
              max-size(int): Maximum number of bytes(Optional).
        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " set" + core_utils.build_cmd_args(kw=kw)
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def enable(self, **kw):
        """
        Enables quota on specific quota type.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              quota-scope(str): Scope for quota in either bucket/user.
              uid(int): uid of an user.
        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        quota_scope = kw.get("quota-scope")
        uid = kw.get("uid")
        cmd = self.base_cmd + f" enable --quota-scope={quota_scope} --uid={uid}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def disable(self, **kw):
        """
        Disables quota on specific quota type.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              quota-scope(str): Scope for quota in either bucket/user.
              uid(int): uid of an user.
        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        quota_scope = kw.get("quota-scope")
        uid = kw.get("uid")
        cmd = self.base_cmd + f" -disable --quota-scope={quota_scope} --uid={uid}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging

import RhcsQeSdk.core.cli.fabfile as fabfile
from RhcsQeSdk.core.utilities import core_utils

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Upgrade:
    """This module provides CLI interface to ceph orch upgrade."""

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " upgrade"

    def start(self, **kw):
        """To start ceph upgrade.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              ceph-version(str): version of ceph to upgrade/downgrade.
              image(str): Image of ceph in registry
        Returns:
          Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " start" + core_utils.build_cmd_args(kw=kw)
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def stop(self, **kw):
        """To stop ceph upgrade.
        Args:
            None
        Returns:
          Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " stop"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def status(self, **kw):
        """To display upgrade status.
        Args:
            None
        Returns:
          Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " status"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def check(self, **kw):
        """To check service versions.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              image_name(str): Image of ceph in registry
        Returns:
          Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        image_name = kw.get("image_name", "")
        cmd = self.base_cmd + f" check {image_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def pause(self, **kw):
        """To pause ceph upgrade.
        Args:
            None
        Returns:
          Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " pause"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def resume(self, **kw):
        """To resume ceph upgrade.
        Args:
            None
        Returns:
          Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " resume"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging

import RhcsQeSdk.core.cli.fabfile as fabfile

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Config:
    """
    This module provides CLI interface to configure cluster via ceph config.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " config"

    def dump(self, **kw):
        """
        Subcommand dump to dump all options for the cluster.

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " dump"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def ls(self, **kw):
        """
        Subcommand ls to list all option names for the cluster.

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " ls"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def help(self, **kw):
        """
        Subcommand help to describe the specified configuration option.

        Args:
            kw(dict) :       Key/value pairs
            Supported keys:
              option(str) : option to be described

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        option = kw.get("option", "")
        cmd = self.base_cmd + f" help {option}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def get_(self, **kw):
        """
        Subcommand get to dump the option(s) for the specified entity.

        Args:
            kw(dict) :       Key/value pairs
            Supported keys:
              who(str) : entity
              option(str) : option to be described

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        who = kw.get("who", "")
        option = kw.get("option", "")
        cmd = self.base_cmd + f" get {who} {option}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def show(self, **kw):
        """
        Subcommand show to display the running configuration of the specified entity.

        Args:
            kw(dict) :       Key/value pairs
            Supported keys:
              who(str) : entity

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        who = kw.get("who", "")
        cmd = self.base_cmd + f" show {who}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def show_with_defaults(self, **kw):
        """
        Subcommand show-with-defaults to display the running configuration along with the
        compiled-in defaults of the specified entity.

        Args:
            kw(dict) :       Key/value pairs
            Supported keys:
              who(str) : entity

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        who = kw.get("who", "")
        cmd = self.base_cmd + f" show-with-defaults {who}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def set_(self, **kw):
        """
        Subcommand set to set an option for one or more specified entities.

        Args:
            kw(dict) :       Key/value pairs
            Supported keys:
              who(str)    : entity
              option(str) : option to be described
              value(str)  :  value for an option

        Examples:
            ceph config set mgr target_max_misplaced_ratio .07
            ceph config set mgr mgr/balancer/sleep_interval 60

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        who = kw.get("who", "")
        option = kw.get("option", "")
        value = kw.get("value", "")
        cmd = self.base_cmd + f" set {who} {option} {value}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rm(self, **kw):
        """
        Subcommand rm to clear an option for one or more entities.

        Args:
            kw(dict) :       Key/value pairs
            Supported keys:
              who(str) : entity
              option(str) : option to be described

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        who = kw.get("who", "")
        option = kw.get("option", "")
        cmd = self.base_cmd + f" rm {who} {option}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def log(self, **kw):
        """
        Subcommand log to show recent history of config changes.

        Args:
            kw(dict) :       Key/value pairs
            Supported keys:
              count(int) : no of recent logs. default is 10

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        count = kw.get("count", 10)
        cmd = self.base_cmd + f" log {count}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def reset(self, **kw):
        """
        Subcommand reset to revert configuration to the specified historical version.

        Args:
            kw(dict) :       Key/value pairs
            Supported keys:
              version(float) : historical version to which configuration is reverted

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        version = kw.get("version")
        cmd = self.base_cmd + f" reset {version}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def assimilate_conf(self, **kw):
        """
        Subcommand assimilate-conf to assimilate options from stdin, and return a new, minimal conf file.

        Args:
            kw(dict) :       Key/value pairs
            Supported keys:
              input_file(str) : Input file
              output_file(str): Output file
        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        input_file = kw.get("input_file")
        output_file = kw.get("output_file")
        cmd = self.base_cmd + f" assimilate-conf -i {input_file} -o {output_file}"
        logger.info(f"Running command {cmd}")

        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def generate_minimal_conf(self, **kw):
        """
        Subcommand generate-minimal-conf to generate a minimal ceph.conf file,
        which can be used for bootstrapping a daemon or a client.

        Args:
            kw(dict) :       Key/value pairs
            Supported keys:
              path(str) : minimal-config-path

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        path = kw.get("path")
        cmd = self.base_cmd + f" generate-minimal-conf > {path}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging

import RhcsQeSdk.core.cli.fabfile as fabfile

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Snap:
    """
    This module provides CLI interface to manage snapshots from images in pool.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " snap"

    def create(self, **kw):
        """
        Creates a snapshot.
        kw(dict): Key/value pairs that needs to be provided to the installer
            Example::
            Supported keys:
                pool_name(str)  : name of the pool
                snap_name(str)  : name of the snapshot
                image_name(str) : name of the image
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        image_name = kw.get("image_name")
        snap_name = kw.get("snap_name")
        pool_name = kw.get("pool_name")
        cmd = self.base_cmd + f" create {pool_name}/{image_name}@{snap_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def ls(self, **kw):
        """
        Lists snapshots.
        Args:
        kw(dict): Key/value pairs that needs to be provided to the installer
            Example::
            Supported keys:
                pool_name(str) : name of the pool
                image_name(str): name of the image
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        image_name = kw.get("image_name")
        pool_name = kw.get("pool_name")
        cmd = self.base_cmd + f" ls {pool_name}/{image_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rollback(self, **kw):
        """
        Rollback a snapshot.
        Args:
        kw(dict): Key/value pairs that needs to be provided to the installer
            Example::
            Supported keys:
                pool_name(str)  : name of the pool
                snap_name(str)  : name of the snapshot
                image_name(str) : name of the image
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        image_name = kw.get("image_name")
        pool_name = kw.get("pool_name")
        snap_name = kw.get("snap_name")
        cmd = self.base_cmd + f" rollback {pool_name}/{image_name}@{snap_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rm(self, **kw):
        """
        Removes the snapshot.
        Args:
        kw(dict): Key/value pairs that needs to be provided to the installer
            Example::
            Supported keys:
                pool_name(str)  : name of the pool
                snap_name(str)  : name of the snapshot
                image_name(str) : name of the image
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        image_name = kw.get("image_name")
        pool_name = kw.get("pool_name")
        snap_name = kw.get("snap_name")
        cmd = self.base_cmd + f" rm {pool_name}/{image_name}@{snap_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def purge(self, **kw):
        """
        Purges the snapshot.
        Args:
        kw(dict): Key/value pairs that needs to be provided to the installer
            Example::
            Supported keys:
                pool_name(str)  : name of the pool
                image_name(str) : name of the image
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        image_name = kw.get("image_name")
        pool_name = kw.get("pool_name")
        cmd = self.base_cmd + f" purge {pool_name}/{image_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def protect(self, **kw):
        """
        Protects the snapshot.
        Args:
        kw(dict): Key/value pairs that needs to be provided to the installer
            Example::
            Supported keys:
                pool_name(str)  : name of the pool
                snap_name(str)  : name of the snapshot
                image_name(str) : name of the image
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        image_name = kw.get("image_name")
        pool_name = kw.get("pool_name")
        snap_name = kw.get("snap_name")
        cmd = self.base_cmd + f" protect {pool_name}/{image_name}@{snap_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def unprotect(self, **kw):
        """
        Unprotects the snapshot.
        Args:
        kw(dict): Key/value pairs that needs to be provided to the installer
            Example::
            Supported keys:
                pool_name(str)  : name of the pool
                image_name(str) : name of the image
                snap_name(str)  : name of the snapshot
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        image_name = kw.get("image_name")
        pool_name = kw.get("pool_name")
        snap_name = kw.get("snap_name")
        cmd = self.base_cmd + f" unprotect {pool_name}/{image_name}@{snap_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging

import RhcsQeSdk.core.cli.fabfile as fabfile

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Caps:
    """
    This module provides CLI interface for commands used for administrative capabilities
    via radosgw-admin caps command.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " caps"

    def add_(self, **kw):
        """Add user capabilities.
        Args:
                uid(str) : User id
                caps(str) : List of caps (e.g., “usage=read, write; user=read”).
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        uid = kw.get("uid", "")
        caps = kw.get("caps", "")
        cmd = self.base_cmd + f" add --uid={uid} --caps={caps}"
        logger.info(f"Running commmand {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rm(self, **kw):
        """Remove user capabilities.
        Args:
                uid(str) : User id
                caps(str) : List of caps (e.g., “usage=read, write; user=read”).
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        uid = kw.get("uid", "")
        caps = kw.get("caps", "")
        cmd = self.base_cmd + f" rm --uid={uid} --caps={caps}"
        logger.info(f"Running commmand {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging

import RhcsQeSdk.core.cli.fabfile as fabfile
from RhcsQeSdk.core.cli.rbd.schedule import Schedule

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


base_cmd_trash = None
schedule_obj = None


class TrashPurge:
    """
    This module provides CLI interface to rbd trash purge.
    """

    global schedule_obj
    schedule = schedule_obj

    def __new__(self, **kw):
        """
        Purges images from the trash.
        Args:
            kw(dict): Key/value pairs that needs to be provided to the installer
            Example::
            Supported keys:
                pool_name(str) : name of the pool
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        pool_name = kw.get("pool_name")
        global base_cmd_trash
        cmd = base_cmd_trash + f" purge {pool_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))


class Purge:
    """
    This module provides abstract CLI interface to rbd trash purge.
    """

    def __new__(self, base_cmd):
        global base_cmd_trash
        base_cmd_trash = base_cmd
        self.base_cmd = base_cmd + " purge"
        global schedule_obj
        schedule_obj = Schedule(self.base_cmd)
        return TrashPurge



import logging

import RhcsQeSdk.core.cli.fabfile as fabfile

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Auth:
    """
    This module provides CLI interface to manage authentication keys via ceph auth.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " auth"

    def add_(self, **kw):
        """
        Subcommand add adds authentication info for a particular entity
        from input file, or random key if no input is given
        and/or any caps specified in the command.

        Args:
                kw(dict): Key/value pairs that needs to be provided to the installer
        Example::
        Supported keys:
              entity (str):      takes the class attribute entity if nothing specified
              caps (dict):       dictionary of capabilties
                                 {
                                    <daemon> : "allow [r|w|x|*|...] [pool={pool-name}] [namespace={namespace-name}"
                                    ...
                                 }
                                 eg:
                                 {
                                    "mon" : "allow r"
                                    "osd" : "allow rw pool=liverpool"
                                 }
              input_file(str):   path to keyring(Optional)

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        entity = kw.get("entity")
        caps_dict = kw.get("caps", {})
        caps = ""
        for daemon, cap in caps_dict.items():
            caps = caps + f" {daemon} {cap}"
        caps.strip()
        input_file = kw.get("input_file")
        if input_file:
            input_file = f"-i {input_file}"
        cmd = self.base_cmd + f" add {entity} {caps} {input_file}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def caps(self, **kw):
        """
        Subcommand caps updates caps for name from caps specified in the command.

        Args:
                kw(dict): Key/value pairs that needs to be provided to the installer
        Example::
        Supported keys:
              entity (str):      takes the class attribute entity if nothing specified
              caps (dict):       dictionary of capabilties
                                 {
                                    <daemon> : "allow [r|w|x|*|...] [pool={pool-name}] [namespace={namespace-name}"
                                    ...
                                 }
                                 eg:
                                 {
                                    "mon" : "allow r"
                                    "osd" : "allow rw pool=liverpool"
                                 }

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        entity = kw.get("entity")
        caps_dict = kw.get("caps", {})
        caps = ""
        for daemon, cap in caps_dict.items():
            caps = caps + f" {daemon} {cap}"
        caps.strip()
        cmd = self.base_cmd + f" caps {entity} {caps}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def del_(self, **kw):
        """
        Subcommand del deletes all caps for name.

        Args:
                kw(dict): Key/value pairs that needs to be provided to the installer
        Example::
        Supported keys:
              entity (str): takes the class attribute entity if nothing specified

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        entity = kw.get("entity")
        cmd = self.base_cmd + f" del {entity}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def export(self, **kw):
        """
        Subcommand export writes keyring for requested entity, or master keyring if none given.

        Args:
                kw(dict): Key/value pairs that needs to be provided to the installer
        Example::
        Supported keys:
              entity (str): takes the class attribute entity if nothing specified

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        entity = kw.get("entity")
        cmd = self.base_cmd + f" export {entity}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def get_(self, **kw):
        """
        Subcommand get writes keyring file with requested key.

        Args:
                kw(dict): Key/value pairs that needs to be provided to the installer
        Example::
        Supported keys:
              entity (str): takes the class attribute entity if nothing specified

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        entity = kw.get("entity")
        cmd = self.base_cmd + f" get {entity}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def get_key(self, **kw):
        """
        Subcommand get writes keyring file with requested key.

        Args:
                kw(dict): Key/value pairs that needs to be provided to the installer
        Example::
        Supported keys:
              entity (str): takes the class attribute entity if nothing specified
              filename(str): Name of keyring file(Optional)

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        entity = kw.get("entity")
        filename = kw.get("filename")
        cmd = self.base_cmd + f" get-key {entity}"
        if filename:
            cmd = cmd + f" -i {filename}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def get_or_create(self, **kw):
        """
        Subcommand get-or-create adds authentication info for a particular entity from input file,
        or random key if no input given and/or any caps specified in the command.
        Subcommand get-or-create-key gets or adds key for name from system/caps pairs specified in the command.
        If key already exists, any given caps must match the existing caps for that key.

        Args:
                kw(dict): Key/value pairs that needs to be provided to the installer
        Example::
        Supported keys:
              entity (str):      takes the class attribute entity if nothing specified
              caps (dict):       dictionary of capabilties
                                 {
                                    <daemon> : "allow [r|w|x|*|...] [pool={pool-name}] [namespace={namespace-name}"
                                    ...
                                 }
                                 eg:
                                 {
                                    "mon" : "allow r"
                                    "osd" : "allow rw pool=liverpool"
                                 }
              input_file(str):   input path to keyring
              output_file(str):  to save the output to a file

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        entity = kw.get("entity")
        caps_dict = kw.get("caps", {})
        caps = ""
        for daemon, cap in caps_dict.items():
            caps = caps + f" {daemon} {cap}"
        caps.strip()
        input_file = kw.get("input_file", "")
        output_file = kw.get("output_file", "")
        if input_file:
            input_file = f"-i {input_file}"
        if output_file:
            output_file = f"-o {output_file}"
        cmd = (
            self.base_cmd + f" get-or-create {entity} {caps} {input_file} {output_file}"
        )
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def import_(self, **kw):
        """
        Subcommand import imports one or more users from the specified keyring.

        Args:
                kw(dict): Key/value pairs that needs to be provided to the installer
        Example::
        Supported keys:
              input_file(str): path to keyring

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        input_file = kw.get("input_file")
        cmd = self.base_cmd + f" import -i {input_file}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def ls(self, **kw):
        """
        Subcommand ls lists out all users in your cluster.

        Args:
            None
        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = "ceph auth ls"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def print_key(self, **kw):
        """
        Subcommand print_key displays requested key to the standard output.

        Args:
                kw(dict): Key/value pairs that needs to be provided to the installer
        Example::
        Supported keys:
              entity (str):      takes the class attribute entity if nothing specified

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        entity = kw.get("entity")
        cmd = self.base_cmd + f" print-key {entity}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import builtins
from keyword import iskeyword

from RhcsQeSdk.core.cli.cli import CLI
from RhcsQeSdk.core.runner.runner import Runner
from RhcsQeSdk.core.utilities.execute_command import ExecuteCommandMixin
from RhcsQeSdk.core.utilities.parallel_executor import ParallelExecutor
from RhcsQeSdk.core.utilities.redefine_args import RedefineArgs
from RhcsQeSdk.core.utilities.workflow import get_module

execute_command_mixin = ExecuteCommandMixin()
METHOD_MAP = dict(
    {
        "must_pass": execute_command_mixin.must_pass,
        "must_fail": execute_command_mixin.must_fail,
    }
)


class ConfigRun:
    def __init__(
        self,
        cluster_name,
        ceph_cluster_dict,
        is_redefine_args=True,
        is_root=False,
    ):
        self.cli_obj = CLI()
        self.cluster_name = cluster_name
        self.ceph_cluster_dict = ceph_cluster_dict
        self.parallel_executor = ParallelExecutor()
        self.fail = False
        self.is_redefine_args = is_redefine_args
        self.is_root = is_root

    def redefine_service(self, method):
        """
        This method is used to rename the inbuilt method by concatenating underscore.
        Args:
          method(str): Takes inbuilt method name as input.

        Returns:
          method(str): Modified method name.

        Example: changes the inbuilt keyword 'import' to 'import_'.
        """
        if "-" in str(method):
            method = str(method).replace("-", "_")
        if self.check_for_builtins(method):
            method = method + "_"
        return method

    def check_for_builtins(self, service):
        """
        This method is used to check whether the service is a inbuilt function.
        If it is an inbuilt function it is modified.
        Args:
          service(str): name of the method

        Returns:
          None
        """
        if iskeyword(service) or service in dir(builtins):
            return True
        else:
            for key in builtins.__dict__.keys():
                if service in dir(builtins.__dict__[key]):
                    return True
            return False

    def get_must_key(self, test_config):
        """
        This method is used to check whether test has to pass or fail or raise an exception mandatorily.
        Args:
          test_config(Dict): dictionary consists of test configuration in key-value pair.
        Returns:
          must_present_key(key): a key [must_pass | must_fail ]
        """
        for must_present_key in METHOD_MAP.keys():
            if test_config.get(must_present_key):
                must_present_key = must_present_key
            else:
                must_present_key = "must_pass"
            return must_present_key

    def run_config(self, config):
        for step in config:
            if step.get("parallel"):
                tasks = []
                for config_pll in step.get("parallel"):
                    tasks.append((self.run_step, config_pll))
                self.parallel_executor.run(tasks)
            if not self.run_step(step):
                break
        if self.fail:
            return False
        else:
            return True

    def run_step(self, step, env_config=None):
        runner = self.build_runner(step, env_config)
        runner.run()
        if runner.step_output.get("status") == "fail":
            self.fail = True
            return False
        return True

    def build_runner(self, step, env_config=None):
        cls = step.get("class")
        method = step.get("method")
        component = step.get("component")
        method = self.redefine_service(method)
        kw = {
            "cluster_name": self.cluster_name,
            "ceph_cluster_dict": self.ceph_cluster_dict,
            "step": step,
        }
        if self.is_redefine_args:
            args = RedefineArgs().get_args(kw=kw)
        else:
            args = step.get("args")
        method = self.redefine_service(method)
        role = step.get("role")
        must_present_key = self.get_must_key(step)
        module_method = get_module(
            obj=self.cli_obj, component=component, module=cls, service=method
        )
        must_method = METHOD_MAP.get(must_present_key)
        step_output = {"status": None, "result": None}
        runner = Runner(
            cluster_name=self.cluster_name,
            roles=role,
            method=module_method,
            must_method=must_method,
            kwargs=args,
            ceph_cluster_dict=self.ceph_cluster_dict,
            parallel=True,
            step_output=step_output,
            is_root=self.is_root,
            env_config=env_config,
        )
        return runner



import logging
from copy import deepcopy

import RhcsQeSdk.core.cli.fabfile as fabfile
from RhcsQeSdk.core.cli.rbd.client import Client
from RhcsQeSdk.core.utilities import core_utils

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Journal:

    """
    This Class provides wrappers for rbd journal commands.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " journal"
        self.client = Client(self.base_cmd)

    def import_(self, **kw):
        """Wrapper for rbd journal import.

        Import image journal.
        Args:
        kw: Key value pair of method arguments
            Example::
            Supported keys:
                path_name(str): import file
                dest_journal_spec(str): destination journal specification.
                                    [<pool-name>/[<namespace>/]]<journal-name>
                path(str): import file
                dest-pool(str): destination pool name
                dest-namespace(str): destination namespace name
                dest(str): destination image name
                dest-journal(str): destination journal name
                verbose(bool): True
                no-error(bool): Continue after error
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")

        kw_copy = deepcopy(kw)
        dest_journal_spec = kw_copy.pop("dest_journal_spec", "")
        path_name = kw_copy.pop("path_name", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)

        cmd = f"{self.base_cmd} import {path_name} {dest_journal_spec}{cmd_args}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def export(self, **kw):
        """Wrapper for rbd journal export.

        Export image journal.
        Args:
        kw: Key value pair of method arguments
            Example::
            Supported keys:
                source_journal_spec(str): source journal specification.
                                    [<pool-name>/[<namespace>/]]<journal-name>
                path_name(str): import file
                namespace(str): name of the namespace
                pool(str): pool name
                image(str): image name
                journal(str): journal name
                path(str): import file
                verbose(bool): True
                no-error(bool): Continue after error
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")

        kw_copy = deepcopy(kw)
        source_journal_spec = kw_copy.pop("source_journal_spec", "")
        path_name = kw_copy.pop("path_name", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)

        cmd = f"{self.base_cmd} export {source_journal_spec} {path_name}{cmd_args}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def info(self, **kw):
        """Wrapper for rbd journal info.

        Show information about image journal.
        Args:
        kw: Key value pair of method arguments
            Example::
            Supported keys:
                journal_spec(str): journal specification.
                                    [<pool-name>/[<namespace>/]]<journal-name>
                pool(str): pool name
                namespace(str): name of the namespace
                image(str): image name
                journal(str): journal name
                format(str): output format (plain, json, or xml) [default: plain]
                pretty-format(bool): True
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")

        kw_copy = deepcopy(kw)
        journal_spec = kw_copy.pop("journal_spec", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)

        cmd = f"{self.base_cmd} info {journal_spec}{cmd_args}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def inspect(self, **kw):
        """Wrapper for rbd journal inspect.

        Inspect image journal for structural errors.
        Args:
        kw: Key value pair of method arguments
            Example::
            Supported keys:
                journal_spec(str): journal specification.
                                    [<pool-name>/[<namespace>/]]<journal-name>
                pool(str): pool name
                namespace(str): name of the namespace
                image(str): image name
                journal(str): journal name
                verbose(bool): True
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")

        kw_copy = deepcopy(kw)
        journal_spec = kw_copy.pop("journal_spec", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)

        cmd = f"{self.base_cmd} inspect {journal_spec}{cmd_args}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def reset(self, **kw):
        """Wrapper for rbd journal reset.

        Reset image journal.
        Args:
        kw: Key value pair of method arguments
            Example::
            Supported keys:
                journal_spec(str): journal specification.
                                    [<pool-name>/[<namespace>/]]<journal-name>
                pool(str): pool name
                namespace(str): name of the namespace
                image(str): image name
                journal(str): journal name
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")

        kw_copy = deepcopy(kw)
        journal_spec = kw_copy.pop("journal_spec", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)

        cmd = f"{self.base_cmd} reset {journal_spec}{cmd_args}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def status(self, **kw):
        """Wrapper for rbd journal status.

        Show status of image journal.
        Args:
        kw: Key value pair of method arguments
            Example::
            Supported keys:
                journal_spec(str): journal specification.
                                    [<pool-name>/[<namespace>/]]<journal-name>
                pool(str): pool name
                namespace(str): name of the namespace
                image(str): image name
                journal(str): journal name
                format(str): output format (plain, json, or xml) [default: plain]
                pretty-format(bool): True
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")

        kw_copy = deepcopy(kw)
        journal_spec = kw_copy.pop("journal_spec", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)

        cmd = f"{self.base_cmd} status {journal_spec}{cmd_args}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import copy
import logging

import RhcsQeSdk.core.cli.fabfile as fabfile
from RhcsQeSdk.core.utilities.metadata import Metadata
from RhcsQeSdk.core.utilities.test_config_run import ConfigRun
from RhcsQeSdk.core.utilities.utils import copy_admin_sshkeys, transfer_remote_files
from RhcsQeSdk.scripts.install_prereq import (
    ConfigureAnsibleInventory,
    ConfigureFirewall,
    SetUpSSHKeys,
)

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


def run(kw):
    test_data = kw.get("test_data")
    cluster_name = kw.get("cluster_name")
    output = {"cluster_names": [], "rc": 1}
    if cluster_name:
        output["cluster_names"].append(cluster_name)
    for step in test_data.get("configs"):
        for role in step.get("role"):
            if len(role.split(":")) == 2:
                output["cluster_names"].append(role.split(":")[0])
    ceph_cluster_dict = kw.get("ceph_cluster_dict")
    metadata = Metadata()
    metadata.ceph_cluster_dict = ceph_cluster_dict
    env_config = {
        "username": "root",
        "password": "passwd",
        "parallel": True,
        "timeout": 300,
    }
    for step in test_data.get("configs"):
        installer = step.get("installer")
        if installer == "cephadm":
            logger.info("Installing prerequisite for cephadm installer")
            ips = []
            nodes = ceph_cluster_dict[cluster_name].get_nodes("installer")
            for node in nodes:
                ips.append(node.ip_address)
            env_config["hosts"] = ips
            cmds = [
                "yum install -y docker",
            ]
            for cmd in cmds:
                if not fabfile.run_command(cmd, env_config):
                    return output
        elif installer == "cephadm-ansible" or None:
            logger.info("Installing prerequisite for cephadm-ansible installer")
            nodes = ceph_cluster_dict[cluster_name].get_nodes()
            ips = []
            for node in nodes:
                ips.append(node.ip_address)
            env_config["hosts"] = ips
            base_path = "/usr/share/cephadm-ansible"
            base_cmd = "ansible-playbook -i"
            for tools_repository in [
                "ansible-2.9-for-rhel-8-x86_64-rpms",
                "rhceph-5-tools-for-rhel-8-x86_64-rpms",
            ]:
                cmd = f"subscription-manager repos --enable={tools_repository}"
                if not fabfile.run_command(cmd, env_config):
                    return output
            cmds = [
                "yum install -y ceph-common",
                "yum install -y cephadm",
                "yum install -y cephadm-ansible",
            ]
            for cmd in cmds:
                if not fabfile.run_command(cmd, env_config):
                    return output
            SetUpSSHKeys().run(
                cluster_name=cluster_name, ceph_cluster_dict=ceph_cluster_dict
            )
            ConfigureFirewall().run(env_config=env_config)
            ConfigureAnsibleInventory().run(
                cluster_name=cluster_name,
                ceph_cluster_dict=ceph_cluster_dict,
                installer=installer,
            )
            env_config_copy = copy.deepcopy(env_config)
            env_config_copy["username"] = "cephuser"
            env_config_copy["password"] = "cephuser"
            nodes = ceph_cluster_dict[cluster_name].get_nodes("installer")
            ips = []
            for node in nodes:
                ips.append(node.ip_address)
            env_config_copy["hosts"] = ips
            fabfile.run_command(
                f"{base_cmd} {base_path}/hosts {base_path}/cephadm-preflight.yml",
                env_config_copy,
            )
            nodes = ceph_cluster_dict[cluster_name].get_nodes("_admin")
            ips = []
            for node in nodes:
                ips.append(node.ip_address)
            env_config["hosts"] = ips
        if not ConfigRun(
            cluster_name, ceph_cluster_dict, is_redefine_args=True, is_root=True
        ).run_step(step, env_config):
            return output
    kw = {
        "cluster_name": cluster_name,
        "env_config": env_config,
        "ceph_cluster_dict": ceph_cluster_dict,
    }
    copy_admin_sshkeys(kw=kw)
    kw = {
        "source_role": f"{cluster_name}:_admin",
        "source_path": "/etc/ceph/ceph.conf",
        "destination_path": "/etc/ceph/ceph.conf",
        "destination_role": f"{cluster_name}:client",
        "cluster_name": cluster_name,
        "env_config": env_config,
        "ceph_cluster_dict": ceph_cluster_dict,
    }
    transfer_remote_files(kw=kw)
    kw["source_path"] = "/etc/ceph/ceph.client.admin.keyring"
    kw["destination_path"] = "/etc/ceph/ceph.client.admin.keyring"
    transfer_remote_files(kw=kw)
    output["rc"] = 0
    return output



from RhcsQeSdk.core.cli.ceph.ceph import Ceph
from RhcsQeSdk.core.cli.cephadm.cephadm import CephAdm
from RhcsQeSdk.core.cli.rados.rados import Rados
from RhcsQeSdk.core.cli.radosgw_admin.radosgw_admin import Radosgw_admin
from RhcsQeSdk.core.cli.rbd.rbd import Rbd


class CLI:
    def __init__(self, base_cmd=""):
        """
        Module for initiaizing root modules for SDK layer.
        """
        self.ceph = Ceph(base_cmd)
        self.rados = Rados(base_cmd)
        self.radosgw_admin = Radosgw_admin(base_cmd)
        self.rbd = Rbd(base_cmd)
        self.cephadm = CephAdm(base_cmd)
        self.rbd_mirror = self.rbd.mirror



import re


class RedefineArgs:
    def __init__(self):
        pass

    def get_args(self, **kw):
        kw = kw.get("kw")
        step = kw.get("step")
        cls = step.get("class").lower()
        method = step.get("method").lower()
        component = step.get("component").lower()
        redefine_method_name = f"{component}_{cls}_{method}"
        args = step.get("args")
        redefine_method = getattr(self, redefine_method_name, None)
        if redefine_method:
            args = redefine_method(kw=kw)
        return args

    def cephadm_cephadm_bootstrap(self, **kw):
        kw = kw.get("kw")
        step = kw.get("step")
        cluster_name = kw.get("cluster_name")
        ceph_cluster_dict = kw.get("ceph_cluster_dict")
        args = step.get("args")
        if args.get("registry-url"):
            args["registry-username"] = "qa@redhat.com"
            args["registry-password"] = "MTQj5t3n5K86p3gH"
        if args.get("mon-ip"):
            ips = [
                ceph_cluster_dict[cluster_name].get_nodes(args.get("mon-ip")).ip_address
            ]
        else:
            ips = []
            nodes = ceph_cluster_dict[cluster_name].get_nodes("mon")
            for node in nodes:
                ips.append(node.ip_address)
        args["mon-ip"] = " ".join(ips)
        return args

    def ceph_orch_apply(self, **kw):
        kw = kw.get("kw")
        step = kw.get("step")
        args = step.get("args")
        service_name = args.get("service_name")
        placement = args.pop("placement", {})
        cluster_name = kw.get("cluster_name")
        ceph_cluster_dict = kw.get("ceph_cluster_dict")
        if placement:
            if "label" in placement:
                args["label"] = placement["label"]
            if "nodes" in placement:
                sep = placement.get("sep", " ")
                nodes = placement.get("nodes", {})
                nodes_list = []
                for node in nodes:
                    for ceph_node in ceph_cluster_dict[cluster_name].get_nodes():
                        searches = re.findall(
                            rf"{node}?\d*", ceph_node.hostname, re.IGNORECASE
                        )
                        for ele in searches:
                            if ele == node:
                                nodes_list.append(ceph_node.hostname)
                nodes_str = f"{sep}".join(nodes_list)
                count = placement.get("count", "")
                if service_name == "rgw" and placement.get("count-per-host", ""):
                    count = placement.get("count-per-host", None)
                nodes_str = f"{count}{sep}{nodes_str}"
                args["nodes_str"] = nodes_str
        return args

    def ceph_host_add(self, **kw):
        kw = kw.get("kw")
        cluster_name = kw.get("cluster_name")
        ceph_cluster_dict = kw.get("ceph_cluster_dict")
        step = kw.get("step")
        args = step.get("args")
        hosts = args.pop("host_name", {})
        for node in ceph_cluster_dict[cluster_name].get_nodes():
            searches = re.findall(rf"{hosts}?\d*", node.hostname, re.IGNORECASE)
            for ele in searches:
                if ele == hosts:
                    hosts = node.hostname
        host_string = ""
        ip_address = [
            ceph_cluster_dict[cluster_name].get_node_by_hostname(hosts).ip_address
        ]
        ip_address = "".join(ip_address)
        host_string = host_string + f"{hosts} {ip_address}"
        args["host_string"] = host_string
        return args

    def ceph_label_add(self, **kw):
        kw = kw.get("kw")
        cluster_name = kw.get("cluster_name")
        ceph_cluster_dict = kw.get("ceph_cluster_dict")
        step = kw.get("step")
        args = step.get("args")
        hosts = args.pop("host_name", {})
        for node in ceph_cluster_dict[cluster_name].get_nodes():
            searches = re.findall(rf"{hosts}?\d*", node.hostname, re.IGNORECASE)
            for ele in searches:
                if ele == hosts:
                    args["hosts"] = node.hostname
        return args



import logging
from copy import deepcopy

import RhcsQeSdk.core.cli.fabfile as fabfile

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Bootstrap:
    """This module provides CLI interface to manage rbd pool peer bootstrap commands."""

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " bootstrap"

    def create(self, **kw):
        """Wrapper for pool peer bootstrap create.

        Create a peer bootstrap token to import in a remote cluster.

        Args:
            kw: Key value pair of method arguments
        Example::
        Supported keys:
            pool_name: Name of the pool of which peer is to be bootstrapped.
            site_name: Name of the site for rbd-mirror
                       name of both sites should be the same
            token_path: path to store bootstrap token
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")

        kw_copy = deepcopy(kw)
        pool_name = kw_copy.pop("pool_name", "")
        token_path = kw_copy.pop("token_path")
        site_name = kw_copy.pop("site_name")

        cmd = (
            f"{self.base_cmd} create {pool_name} --site-name {site_name} > {token_path}"
        )
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def import_(self, **kw):
        """Wrapper for pool peer bootstrap import.

        Import a peer bootstrap token created from a remote cluster

        Args:
            kw: Key value pair of method arguments
        Example::
        Supported keys:
            pool_name: Name of the pool of which peer is to be bootstrapped.
            site_name: Name of the site for rbd-mirror
                       name of both sites should be the same
            token_path: The file name in which the key is stored
            direction: mirroring direction (rx-only, rx-tx)
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")

        kw_copy = deepcopy(kw)
        pool_name = kw_copy.pop("pool_name", "")
        site_name = kw_copy.pop("site_name")
        direction = kw_copy.pop("direction")
        token_path = kw_copy.pop("token_path")

        cmd = f"{self.base_cmd} import {pool_name} --site-name {site_name} --direction {direction} {token_path}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)

file_handler = logging.FileHandler("startup.log", mode="a")
file_handler.setLevel(logging.ERROR)

file_handler.setFormatter(formatter)
logger.addHandler(file_handler)


class Runner:
    """Purpose of this module is to provide environment information such as clustername, hosts, nodename, roles etc
    to run ceph cli commands.
    """

    def __init__(
        self,
        cluster_name,
        roles,
        method,
        must_method,
        kwargs,
        ceph_cluster_dict,
        parallel,
        step_output,
        is_root=False,
        env_config=None,
    ):
        self.step_output = step_output
        self.ceph_cluster_dict = ceph_cluster_dict
        self.method = method
        self.kwargs = kwargs
        self.roles = roles
        self.parallel = parallel
        if not self.roles:
            self.roles = ["client"]
        self.must_method = must_method
        self.cluster_name = cluster_name
        ips = []
        for role in roles:
            if len(role.split(":")) == 2:
                cluster_and_role = role.split(":")
                cluster_name = cluster_and_role[0]
                role = cluster_and_role[1]
            else:
                cluster_name = self.cluster_name
                role = role
            nodes = ceph_cluster_dict[cluster_name].get_nodes(role)
            for node in nodes:
                ips.append(node.ip_address)
        self.hosts = ips
        self.cluster_name = cluster_name
        self.is_root = is_root
        self.env_config = env_config
        self.set_environment_for_run()

    def set_environment_for_run(self):
        """
        Sets environment variables to run a command through fabfile.
        Args:
            None

        Returns:
            None
        """
        if not self.env_config:
            self.env_config = {
                "hosts": self.hosts,
                "username": "cephuser",
                "password": "cephuser",
                "parallel": self.parallel,
            }
        if self.is_root:
            self.env_config["username"] = "root"
            self.env_config["password"] = "passwd"

    def run(self):
        """
        Runs a command in must methods[must_pass | must_fail | must_raise].
        Args:
            None

        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        out = self.must_method(
            function=self.method,
            kw_args=self.kwargs,
            env_config=self.env_config,
            cluster_name=self.cluster_name,
            step_output=self.step_output,
        )
        return out



import logging
import time

import yaml

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)


class RetryMixin:
    def __init__(self):
        pass

    def compare_result_with_args(self, output, args):
        """
        compare output with the arguments and returns output matches or not.
        Args:
          output(dict): results of all ip's.
          args(dict): key val pair that needs to be checked with the output.

        Returns:
          True or False.
        """
        success = True
        for ip, result in output.items():
            out_dict = yaml.safe_load(result)
            out_dict = list(out_dict.values())[0]
            if args.get("value") not in out_dict.get(args.get("key")):
                success = False
                break
        return success

    def dict_check(self, result, args):
        """
        compare result with the arguments and returns output matches or not.
        Args:
          result(dict): result of the command.
          args(dict): key val pair that needs to be checked with the result.

        Returns:
          True or False.
        """
        if not isinstance(result, dict):
            return True
        for key, val in result.items():
            if key == args.get("key"):
                if not val == args.get("val"):
                    return False
            elif not self.dict_check(val, args):
                return False
        return True

    def retry(self, kw):
        success = 0
        runner = kw.get("runner")
        retry_args = kw.get("args")
        interval_seconds = retry_args.get("interval_seconds", 1)
        pattern = retry_args.get("pattern")
        for i in range(retry_args.get("max_attempts", 1)):
            out = runner.run()

            if self.compare_result_with_args(out, pattern):
                success = 1
                break
            time.sleep(interval_seconds)
        if not success:
            logger.info("status not equals to expected even after max attempts")
            return False
        return True



import logging

from RhcsQeSdk.core.cli.ceph.orch.rm import Rm

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Osd:
    """This module provides CLI interface to ceph orch osd."""

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " osd"
        self.rm = Rm(self.base_cmd)



import logging

import RhcsQeSdk.core.cli.fabfile as fabfile

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Application:
    """This module provides CLI interface to add annotation to a pool via ceph osd pool application."""

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " application"

    def disable(self, **kw):
        """
        The disable command will disable a client application from conducting I/O operations on a pool.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:

                pool_name(str): name of the pool.
                app(str): component of ceph (cephfs, rgw, rbd).
                yes-i-really-mean-it(bool): whether to disable or not(Optional)

        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        pool_name = kw.get("pool_name", "ceph")
        app = kw.get("app", "")
        ending = kw.get("--yes-i-really-mean-it", "")
        cmd = f"{self.base_cmd} disable {pool_name} {app} {ending}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def set_(self, **kw):
        """
        To set client application metadata on a pool.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:

                pool_name(str): name of the pool.
                app(str): component of ceph (cephfs, rbd, rgw).
                key(Union[str,int]): name of the property of an osd pool.
                value(Union[str,int]): value to be set against the key.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        pool_name = kw.get("pool_name", "ceph")
        app = kw.get("app", "")
        key = kw.get("key", None)
        value = kw.get("value", None)
        cmd = f"{self.base_cmd} set {pool_name} {app} {key} {value}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rm(self, **kw):
        """
        Removes the key-value pair for the given key in the given application of the
        given pool.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:

                pool_name(str): name of the pool.
                app(str): component of ceph (cephfs, rbd, rgw).
                key(Union[str,int]): name of the property of an osd pool.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        pool_name = kw.get("pool_name", "ceph")
        app = kw.get("app", "")
        key = kw.get("key", None)
        cmd = f"{self.base_cmd} rm {pool_name} {app} {key}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def get_(self, **kw):
        """
        Displays the value for the given key that is associated with the given
        application of the given pool.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:

                pool_name(str): name of the pool.
                app(str): component of ceph (cephfs, rbd, rgw).
                key(Union[str,int]): name of the property of an osd pool.

        Returns:
            Dict(str):
            A mappping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        pool_name = kw.get("pool_name", "ceph")
        app = kw.get("app", "")
        key = kw.get("key", None)
        cmd = f"{self.base_cmd} get {pool_name} {app} {key}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def enable(self, **kw):
        """
        The enable command will enable a client application to conduct I/O operations on a pool.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:

                pool_name(str): name of the pool.
                app(str): component of ceph (cephfs, rgw, rbd).
                yes-i-really-mean-it(bool): whether to disable or not(Optional)

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for taht host's execution run.
        """
        kw = kw.get("kw")
        pool_name = kw.get("pool_name", "ceph")
        app = kw.get("app", "")
        ending = kw.get("yes-i-really-mean-it", "")
        cmd = f"{self.base_cmd} enable {pool_name} {app} {ending}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging
from copy import deepcopy

import RhcsQeSdk.core.cli.fabfile as fabfile
from RhcsQeSdk.core.cli.rbd.purge import Purge
from RhcsQeSdk.core.utilities import core_utils

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Trash:
    """This module provides CLI interface to manage block device images from/in trash."""

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " trash"
        self.purge = Purge(self.base_cmd)

    def mv(self, **kw):
        """
        Moves an image to the trash.
        Args:
            kw(dict): Key/value pairs that needs to be provided to the installer
            Example::
            Supported keys:
                image_name(str) :  name of image to be moved to trash
                pool_name(str)  : name of the pool
                expires_at(str) : Expiration time to defer the image (optional)
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        kw_copy = deepcopy(kw)
        image_name = kw_copy.pop("image_name", "")
        pool_name = kw_copy.pop("pool_name", "")
        cmd = (
            self.base_cmd
            + f" mv {pool_name}/{image_name}"
            + core_utils.build_cmd_args(kw=kw_copy)
        )
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rm(self, **kw):
        """
        Removes an image from the trash.
        Args:
            kw(dict): Key/value pairs that needs to be provided to the installer
            Example::
            Supported keys:
                image_id(int)  :  id of image to be removed from trash
                pool_name(str) : name of the pool
                force(bool)    :  To determine whether its a forced operation or not(optional)
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        kw_copy = deepcopy(kw)
        image_id = kw_copy.pop("image_id", "")
        pool_name = kw_copy.pop("pool_name", "")
        cmd = (
            self.base_cmd
            + f" rm {pool_name}/{image_id}"
            + core_utils.build_cmd_args(kw=kw_copy)
        )
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def restore(self, **kw):
        """
        Restores an image from the trash.
        Args:
            kw(dict): Key/value pairs that needs to be provided to the installer
            Example::
            Supported keys:
                image_id(int) :  id of image to be restored from trash
                pool_name(str) : name of the pool
                image(int) :  name of image to be renamed(Optional)
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        kw_copy = deepcopy(kw)
        image_id = kw_copy.pop("image_id", "")
        pool_name = kw_copy.pop("pool_name", "")
        cmd = (
            self.base_cmd
            + f" restore {pool_name}/{image_id}"
            + core_utils.build_cmd_args(kw=kw_copy)
        )
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def ls(self, **kw):
        """
        Lists imageids from the trash.
        Args:
            kw(dict): Key/value pairs that needs to be provided to the installer
            Example::
            Supported keys:
                pool_name(str) : name of the pool
                all(bool) : whether to list all trashed images(Optional)
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        kw_copy = deepcopy(kw)
        pool_name = kw_copy.pop("pool_name", "")
        cmd = self.base_cmd + f" ls {pool_name}" + core_utils.build_cmd_args(kw=kw_copy)
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging

import RhcsQeSdk.core.cli.fabfile as fabfile

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Period:
    """
    This module provides CLI interface for period management for object gateway service.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " period"

    def rm(self, **kw):
        """
        This method is used to remove a period.
        Args:
          period(str) : Period Id of a specific period.

        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        period_id = kw.get("period")
        cmd = self.base_cmd + f" rm --period={period_id}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def get_(self, **kw):
        """
        This method is used to get the info of specific period.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              period(str) : Period Id of a specific period.

        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        period_id = kw.get("period")
        cmd = self.base_cmd + f" get --period={period_id}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def get_current(self, **kw):
        """
        This method is used to get the current period info.
        Args:
          None

        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " get-current"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def pull(self, **kw):
        """
        This method is used to pull a period.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              url(str) : url to the master zone.
              access-key(str) : The secondary zones will require them to authenticate with the master zone.
              secret-key(str): The secondary zones will require them to authenticate with the master zone.

        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        url = kw.get("url")
        access_key = kw.get("access-key")
        secret_key = kw.get("secret-key")
        cmd = (
            self.base_cmd
            + f" pull --url={url} --access-key={access_key} --secret={secret_key}"
        )
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def push(self, **kw):
        """
        This method is used to push a period.
        Args:
          None

        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " push"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def list_(self, **kw):
        """
        This method is used to list all periods.
        Args:
          None

        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " list"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def update_(self, **kw):
        """
        This method is used to update the staging period.
        Args:
          None

        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " update"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def commit(self, **kw):
        """
        This method is used to commit the staging period.
        Args:
          rgw-realm(str): Takes the realm name as input.

        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        rgw_realm = kw.get("rgw-realm")
        cmd = self.base_cmd + f" update --rgw-realm={rgw_realm} --commit"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



from RhcsQeSdk.core.utilities.metadata import Metadata
from RhcsQeSdk.core.utilities.test_config_run import ConfigRun


def run(kw):
    test_data = kw.get("test_data")
    cluster_name = kw.get("cluster_name")
    output = {"cluster_names": [], "rc": 1}
    if cluster_name:
        output["cluster_names"].append(cluster_name)
    for step in test_data.get("configs"):
        for role in step.get("role"):
            if len(role.split(":")) == 2:
                output["cluster_names"].append(role.split(":")[0])
    ceph_cluster_dict = kw.get("ceph_cluster_dict")
    metadata = Metadata()
    metadata.ceph_cluster_dict = ceph_cluster_dict
    if not ConfigRun(cluster_name, ceph_cluster_dict, is_root=True).run_config(
        test_data.get("configs")
    ):
        return output
    output["rc"] = 0
    return output
